/* gf2x/gf2x-config-export.h.  Generated from gf2x-config-export.h.in by configure.  */
/* gf2x/gf2x-config-export.h.in ; this file is *NOT* automatically
 * generated. It is not the same as gf2x/gf2x-config.h.in :
 *
 *  - gf2x/gf2x-config.h.in is relevant for internal compilation units
 *    within the library.
 *
 *  - gf2x/gf2x-config-export.h.in (this file) is potentially exposed to
 *    end users, because it modifies the inlines which are exposed
 *    through the gf2x-small.h file.
 */

/* Define if pclmul as present in the source tree is supported by the compiler
   and hardware */
#ifndef GF2X_HAVE_PCLMUL_SUPPORT
/* #undef GF2X_HAVE_PCLMUL_SUPPORT */
#endif

/* Define if sse-2 code as present in the source tree is supported by the
   compiler */
#ifndef GF2X_HAVE_SSE2_SUPPORT
#define GF2X_HAVE_SSE2_SUPPORT 1
#endif

/* Define to 1 if the mpir.h header exists */
#ifndef GF2X_HAVE_MPIR_H
/* #undef GF2X_HAVE_MPIR_H */
#endif

